@@ -55,10 +55,12 @@ delete_isoforms <- function(alignment) {
   }
   new = list()
   if (length(lines_to_delete >= 1)) {
-    new$nb = alignment$nb - (length(lines_to_delete))
-    new$nam = alignment$nam[-lines_to_delete]
-    new$seq = alignment$seq[-lines_to_delete]
-    output = new
+    alignment$nb = alignment$nb - (length(lines_to_delete))
+    alignment$nam = alignment$nam[-lines_to_delete]
+    alignment$seq = alignment$seq[-lines_to_delete]
+    output = seqinr::as.alignment(nb = alignment$nb,nam = alignment$nam, seq = alignment$seq,com=NA )
+    no_deleted = length(lines_to_delete)
+    warning(paste(no_deleted,"isoforms were deleted"))
   }
   else{
     output = alignment
@@ -110,7 +112,7 @@ cons2seqs_ident <-  function(alignment, consensus_seq) {
   #consensus_seq - calculated consensus (output of consensusus())
   true_percentage = c()
 
-  for (i in seq(1, alignment_parameters(alignment)$row_no)) {
+  for (i in seq(1, align_params(alignment)$row_no)) {
     true_percentage[i] = length(which((
       consensus_seq == seqinr::s2c(alignment$seq[i])
     ) == TRUE)) / length(consensus_seq)
@@ -118,7 +120,7 @@ cons2seqs_ident <-  function(alignment, consensus_seq) {
   }
   return(true_percentage)
 }
-alignment_parameters <- function(alignment) {
+align_params <- function(alignment) {
   #alignment data
   #return list of alignment size [row_numbers, col_numbers]
   aligned_sequences = alignment$seq
@@ -128,7 +130,7 @@ alignment_parameters <- function(alignment) {
   return(param)
 }
 
-aligned_sequences_matrix2groups <-
+align_seq_mtx2grs <-
   function(aligned_sequences_matrix,
            grouping_method) {
     rows = dim(aligned_sequences_matrix)[1]
@@ -292,12 +294,12 @@ cons2seqs_sim <-
 alignment2matrix <- function(alignment) {
   #alignment data
   #returns alignment as a matrix
-  prmt = alignment_parameters(alignment = alignment)
+  prmt = align_params(alignment = alignment)
   aligned_sequences_matrix = matrix("-", prmt$row_no, prmt$col_no)
 
   for (i in seq(1, prmt$row_no)) {
     #Putting aligned seqs into matrix
-    temp = seqinr::s2c(alignment$seq[i])
+    temp = toupper(seqinr::s2c(alignment$seq[i]))
     for (j in seq(1, prmt$col_no)) {
       aligned_sequences_matrix[i, j] = temp[j]
 
@@ -310,15 +312,22 @@ calculate_AA_variation <-
   function(alignment,
            threshold = NULL,
            grouped = F,
-           grouping_method = "substitution_matrix") {
+           grouping_method = "substitution_matrix",
+           weights = NULL,
+           pseudo_counts = F) {
     #prmt- size of alignment (output of get_parameter())
     #sequence_alignment-file[[3]]
     #threshold-threshold for detecting key amino acids (the percentage of all at the given position)
     #returns list of matrices with tabelarised symbols of the most common AA in alignment column and percentage values for contributed AA
-    prmt = alignment_parameters(alignment = alignment)
-
+    freq=c()
+    prmt = align_params(alignment = alignment)
+    if(pseudo_counts){
+      grouped = F
+      weights = NULL
+    }
     if (is.null(threshold)) {
-      keyaas_treshold = 1 / prmt$row_no
+      # keyaas_treshold = 1 / prmt$row_no
+      keyaas_treshold = 0.0000001
     } else{
       if (threshold <= 0) {
         stop("The threshold must be > 0! Do not specify any to inspect all the residues in MSA.")
@@ -329,19 +338,44 @@ calculate_AA_variation <-
 
     aligned_sequences_matrix = alignment2matrix(alignment = alignment)
     if (grouped == T) {
-      aligned_sequences_matrix = aligned_sequences_matrix2groups(aligned_sequences_matrix, grouping_method = grouping_method)
+      aligned_sequences_matrix = align_seq_mtx2grs(aligned_sequences_matrix, grouping_method = grouping_method)
     }
-    keyaas = matrix("n", dim(aligned_sequences_matrix)[2], 20 * 2)
-    keyaas_per = matrix("n", dim(aligned_sequences_matrix)[2], 20 * 2)
-
+    keyaas = matrix("n", dim(aligned_sequences_matrix)[2], 21 * 2)
+    keyaas_per = matrix("n", dim(aligned_sequences_matrix)[2], 21 * 2)
+    pc = calculate_pseudo_counts(alignment)
     for (i in seq(1, dim(aligned_sequences_matrix)[2])) {
-      table = (sort(table(aligned_sequences_matrix[, i]), decreasing = T))
+      if (length(table(aligned_sequences_matrix[, i])) == 1) {
+        x = append(aligned_sequences_matrix[, i], "X")
+      } else{
+        x = aligned_sequences_matrix[, i]
+      }
+      table = (sort(table(x), decreasing = T))
+      if (pseudo_counts) {
+        df_table = as.data.frame(table)
+        names(df_table) = c("AAs", "freq")
+        pseudo_counts = pc[, i]
+        df_pc = as.data.frame(pseudo_counts)
+        df_pc = dplyr::mutate(df_pc, AAs = rownames(df_pc))
+        df_merged_table = df_pc %>% dplyr::full_join(df_table)
+        df_merged_table[is.na(df_merged_table)] = 0
+        df_merged_table = df_merged_table %>% dplyr::mutate(Frequency = pseudo_counts +
+                                                       freq) %>% dplyr::select(AAs, Frequency)
+      } else{
+        df_merged_table = as.data.frame(table)
+        names(df_merged_table) = c("AAs", "Frequency")
+      }
       #AA types and frequencies
       aas = names(table)
+      over_threshold = which(df_merged_table$Frequency > keyaas_treshold)
+      AAs = df_merged_table$AAs[over_threshold]
+      Frequency = df_merged_table$Frequency[over_threshold]
+      order_decresasing = order(Frequency, decreasing = T)
+      Frequency = Frequency[order_decresasing]
+      AAs = AAs[order_decresasing]
 
-      keyaas[i, 1:length(matrix(aas[which(table >= keyaas_treshold)], 1, length(aas[which(table >= keyaas_treshold)])))] = matrix(aas[which(table >= keyaas_treshold)], 1, length(aas[which(table >= keyaas_treshold)]))
+      keyaas[i, 1:length(matrix(AAs, 1, length(AAs)))] = matrix(AAs, 1, length(AAs))
       #if there are any AA (=always) then these are introduced to the keyaas matrix
-      keyaas_per[i, 1:length(matrix((table)[which(table >= keyaas_treshold)], 1, length((table)[which(table >= keyaas_treshold)])))] = matrix(round((table)[which(table >= keyaas_treshold)] / prmt$row_no, 3) * 100, 1, length(aas[which(table >= keyaas_treshold)]))
+      keyaas_per[i, 1:length(matrix(Frequency, 1, length(Frequency)))] = matrix(round(Frequency / sum(Frequency), 6) * 100, 1, length(AAs))
       #similarly in the percentages case
     }
     i = 1
@@ -353,12 +387,34 @@ calculate_AA_variation <-
     #transpose matrix
     keyaas_per = t(keyaas_per[, 1:i]) #transpose matrix
     #merging key AAs symbols table with key AAs percentages table
+
     size = dim(keyaas)
     output = matrix("-", size[1] * 2, size[2])
+    if (!is.null(weights)) {
+      if (length(weights) != prmt$row_no) {
+        stop("The length of weights vector must equal the number of sequences in the alignment!")
+      } else{
+        weight = matrix("n",
+                        ncol =  dim(aligned_sequences_matrix)[2],
+                        nrow =  21)
+        weights = weights / mean(weights)
+        for (i in seq_len(ncol(aligned_sequences_matrix))) {
+          representatives = unique(keyaas[, i])
+          representatives = representatives[!representatives == "n"]
+          for (j in seq_len(length(representatives))) {
+            which_representative = which(aligned_sequences_matrix[, i] == representatives[j])
+            weight[j, i] = mean(weights[which_representative])
+          }
+          keyaas_per[, i] = as.numeric(keyaas_per[, i]) * as.numeric(weight[, i])
+        }
+
+      }
+    }
+
     j = 1
     for (i in seq(1, size[1] * 2, 2)) {
-      output[i,] = keyaas[j,]
-      output[i + 1,] = keyaas_per[j,]
+      output[i, ] = keyaas[j, ]
+      output[i + 1, ] = keyaas_per[j, ]
       j = j + 1
     }
 
@@ -366,9 +422,10 @@ calculate_AA_variation <-
       AA = keyaas,
       percentage = keyaas_per,
       matrix = output
+      #weights = weight
     ))
   }
-noteworthy_sequences <- function(percentage, alignment) {
+noteworthy_seqs <- function(percentage, alignment) {
   max = which.max(percentage)
   namelist = alignment[[2]]
   out.max = list(c(namelist[max], max)) #output is a name of sequence and position in alignment
@@ -494,7 +551,7 @@ find_seq <- function(sequence_id, alignment) {
 }
 
 
-exclude_low_probability_structures <-
+excl_low_prob_strcts <-
   function(structure, threshold) {
     if (length(structure) == 3) {
       for (i in seq(1, dim(structure[[3]])[1], by = 1)) {
@@ -600,21 +657,23 @@ create_final_CSV <-
       )
     }
     if (Sys.info()[[1]] == "Linux") {
-      print(paste("Output written to: ", getwd(), "/", filename, ".csv", sep = ""))
+      warning(paste("Output written to: ", getwd(), "/", filename, ".csv", sep = ""))
     }
     return(final_output)
   }
 
 Escore_conservativity <-
-  function(alignment, grouping_method = NULL) {
+  function(alignment, grouping_method = NULL, weights = NULL,pseudo_counts = F) {
     if (is.null(grouping_method)) {
-      var_aa = calculate_AA_variation(alignment, threshold = 0.01)
+      var_aa = calculate_AA_variation(alignment, threshold = 0.01, weights = weights,pseudo_counts = pseudo_counts)
     } else {
       var_aa = calculate_AA_variation(
         alignment,
         threshold = 0.01,
         grouped = T,
-        grouping_method = grouping_method
+        grouping_method = grouping_method,
+        weights = weights,
+        pseudo_counts = F
       )
     }
     max_cons = c()
@@ -646,15 +705,17 @@ Escore_conservativity <-
     return(return_data)
   }
 
-kabat_conservativity <- function(alignment) {
+kabat_conservativity <- function(alignment,weights=NULL,pseudo_counts=F) {
   if (!is.matrix(alignment)) {
     aligned_sequences_matrix = alignment2matrix(alignment = alignment)
-  }
-  else{
+  }else{
     aligned_sequences_matrix = alignment
   }
   Kabat = rep(NaN, dim(aligned_sequences_matrix)[2])
+
+  var_aa = calculate_AA_variation(alignment,weights = weights,pseudo_counts = pseudo_counts)
   for (rep in seq(1, dim(aligned_sequences_matrix)[2], 1)) {
+
     column = aligned_sequences_matrix[, rep]
     #KABAT
     K = c()
@@ -664,18 +725,30 @@ kabat_conservativity <- function(alignment) {
     N = length(column)
     #no of objects
     K = length(as.numeric(table(column)))
+    if(!is.null(weights)){
+      aas = table(column)
+      for(i in length(aas)){
+        match = which(var_aa$AA[,rep] == names(aas)[i])
+        aas[i] = aas[i] * as.numeric(var_aa$weights[match,rep])
+      }
+    } else{
+      AAs=var_aa$AA[,rep]
+      PER=suppressWarnings(as.numeric(var_aa$percentage[,rep]))
+      per = PER[!is.na(PER)]
+      aas = AAs[!is.na(PER)]
+      names(per) = aas
+      table=per
+    }
     #no of classes
-    n1 = as.numeric(sort(table(column))[length(table(column))])
-
+    n1 = as.numeric(sort(table)[length(table)])
     Kabat[rep] = (K / n1) * N
-
   }
   Kabat_entropy_normalized = Kabat / max(Kabat)
 
   return(Kabat_entropy_normalized)
 }
 
-schneider_conservativity <- function(alignment) {
+schneider_conservativity <- function(alignment,weights=NULL,pseudo_counts=F) {
   if (!is.matrix(alignment)) {
     aligned_sequences_matrix = alignment2matrix(alignment = alignment)
   }
@@ -684,10 +757,24 @@ schneider_conservativity <- function(alignment) {
   }
   symbols = 21
   sum_schneider = rep(NaN, dim(aligned_sequences_matrix)[2])
+  var_aa = calculate_AA_variation(alignment, weights = weights,pseudo_counts = pseudo_counts)
   for (rep in seq(1, dim(aligned_sequences_matrix)[2], 1)) {
     column = aligned_sequences_matrix[, rep]
-    #SCHNEIDER, SHANNON
-    tab = as.numeric(table(column))
+    if(!is.null(weights)){
+      aas = table(column)
+      for(i in length(aas)){
+        match = which(var_aa$AA[,rep] == names(aas)[i])
+        aas[i] = aas[i] * as.numeric(var_aa$weights[match,rep])
+      }
+    } else{
+      AAs=var_aa$AA[,rep]
+      PER=suppressWarnings(as.numeric(var_aa$percentage[,rep]))
+      per = PER[!is.na(PER)]
+      aas = AAs[!is.na(PER)]
+      names(per) = aas
+      table=per
+    }
+    tab = as.numeric(table)
     K = c()
     p = c()
     n = c()
@@ -709,10 +796,10 @@ schneider_conservativity <- function(alignment) {
     sum_schneider[rep] = -sum_internal_schneider
 
   }
-  return(sum_schneider)
+  return(sum_schneider/max(sum_schneider))
 }
 
-shannon_conservativity <- function(alignment) {
+shannon_conservativity <- function(alignment, weights=NULL,pseudo_counts=F) {
   if (!is.matrix(alignment)) {
     aligned_sequences_matrix = alignment2matrix(alignment = alignment)
   }
@@ -720,10 +807,25 @@ shannon_conservativity <- function(alignment) {
     aligned_sequences_matrix = alignment
   }
   sum = rep(NaN, dim(aligned_sequences_matrix)[2])
+  var_aa = calculate_AA_variation(alignment, threshold = 0.01, weights = weights,pseudo_counts = pseudo_counts)
+
   for (rep in seq(1, dim(aligned_sequences_matrix)[2], 1)) {
     column = aligned_sequences_matrix[, rep]
-    #SCHNEIDER, SHANNON
-    tab = as.numeric(table(column))
+    if(!is.null(weights)){
+      aas = table(column)
+      for(i in length(aas)){
+        match = which(var_aa$AA[,rep] == names(aas)[i])
+        aas[i] = aas[i] * as.numeric(var_aa$weights[match,rep])
+      }
+    } else{
+      AAs=var_aa$AA[,rep]
+      PER=suppressWarnings(as.numeric(var_aa$percentage[,rep]))
+      per = PER[!is.na(PER)]
+      aas = AAs[!is.na(PER)]
+      names(per) = aas
+      table=per
+    }
+    tab = as.numeric(table)
     K = c()
     p = c()
     n = c()
@@ -742,9 +844,139 @@ shannon_conservativity <- function(alignment) {
     }
     sum[rep] = -sum_internal
   }
-  return(sum)
+  return(sum/max(sum))
 }
 
+pairwise_alignment_MSA <- function(alignment) {
+  #define a function to use in sapply
+  convert_then_align <- function(seqj, seqi) {
+    seqi = seqinr::s2c(seqi)
+    seqi = seqinr::c2s(seqi[seqi != "-"])
+    seqj = seqinr::s2c(seqj)
+    seqj = seqinr::c2s(seqj[seqj != "-"])
+    return(Biostrings::pairwiseAlignment(seqi, seqj, scoreOnly = T))
+  }
+
+  score_mtx = matrix(NA, nrow = alignment$nb, ncol = alignment$nb)
+  for (i in seq_len(alignment$nb)) {
+    seqi = alignment$seq[i]
+    score_mtx[i,] = sapply(alignment$seq, convert_then_align, seqi)
+    print(i)
+  }
+  return(score_mtx)
+}
+
+preprocess_hmm_output <- function(hmm_out){
+  size = dim(hmm_out)
+  hmm_out = hmm_out[seq(from = 1,to = size[1]-1,by = 3),-1]
+  sites = as.numeric(unlist(hmm_out[,21]))
+  probs = hmm_out[,1:20]
+  #calculate probabilities; prob = -log(x)
+  probs = exp(-probs)
+  return(list(probabilities=probs,alignment_positions=sites))
+}
+
+CRE_conservativity <- function(alignment, hmmbuild_path=NULL, pairwiseAlignemnt_scores=NULL) {
+
+  if(is.null(pairwiseAlignemnt_scores)){
+    pairwiseAlignemnt_scores=pairwise_alignment_MSA(alignment)
+  }
+  #Make sure the hmmbuild is there
+  if(.Platform$OS.type == "unix") {
+    if(is.null(hmmbuild_path)){
+      hmmbuild_path = system("which hmmbuild",intern = T)
+    }
+    if(length(hmmbuild_path)==0){
+      stop("You need to specify the hmmbuild path as it cannot be located in your system")
+    }
+  } else {
+    if(is.null(hmmbuild_path)){
+      stop("You need to specify the hmmbuild path as it cannot be located in your system")
+    }
+  }
+
+  #perform hierarchical clustering on the distance matrix obtained from pairwise alignemnt matrix
+  dendro = stats::hclust(stats::dist(pairwiseAlignemnt_scores), method = "average")
+  #cut the tree and get clusters
+  clusters = stats::cutree(dendro, h = 8000)
+  #get the number of clusters
+  no_clusters = max(clusters)
+  RE = matrix(data = NA,
+              nrow = no_clusters,
+              ncol = length(seqinr::s2c(alignment$seq[1])))
+  CRE = c()
+  Z = c()
+  #iterate over each clusters
+  for (i in seq_len(no_clusters)) {
+    #create copies of the alignemnt object to work on
+    sub_alignment = alignment
+    leftover_alignment = alignment
+    #get the indices of the sequences in the cluster of interest
+    to_delete = which(clusters == clusters[i])
+    if(length(to_delete)>1){
+      #delete appropriate sequences and their names in the objects and adjust the sequeces counts
+      sub_alignment$seq = sub_alignment$seq[to_delete]
+      leftover_alignment$seq = leftover_alignment$seq[-to_delete]
+      sub_converted_seq = lapply(sub_alignment$seq, seqinr::s2c)
+      leftover_converted_seq = lapply(leftover_alignment$seq, seqinr::s2c)
+      sub_alignment$nam = sub_alignment$nam[to_delete]
+      leftover_alignment$nam = leftover_alignment$nam[-to_delete]
+      leftover_alignment$nb = leftover_alignment$nb - length(to_delete)
+      sub_alignment$nb = length(to_delete)
+      #write updated objects to files
+      leftover_fasta_name = "leftover_MSA.fasta"
+      sub_fasta_name = "sub_MSA.fasta"
+      seqinr::write.fasta(sequences = leftover_converted_seq,
+                  names = leftover_alignment$nam,
+                  file.out = leftover_fasta_name)
+      seqinr::write.fasta(sequences = sub_converted_seq,
+                  names = sub_alignment$nam,
+                  file.out = sub_fasta_name)
+      #prepare commands to run HMMER
+      sub_hmm_command = paste(hmmbuild_path, "sub_hmm.out", sub_fasta_name)
+      leftover_hmm_command = paste(hmmbuild_path, "leftover_hmm.out", leftover_fasta_name)
+      system(command = sub_hmm_command, wait = T)
+      system(command = leftover_hmm_command, wait = T)
+      #read the HMMER outputs in
+      leftover_hmm <-
+        readr::read_table("leftover_hmm.out",
+                    col_names = FALSE,
+                    skip = 21)
+      sub_hmm <-
+        readr::read_table("sub_hmm.out", col_names = FALSE, skip = 21)
+      #preprocess the data
+      leftover_prob = preprocess_hmm_output(leftover_hmm)$probabilities
+      leftover_pos = preprocess_hmm_output(leftover_hmm)$alignment_positions
+      sub_prob = preprocess_hmm_output(sub_hmm)$probabilities
+      sub_pos = preprocess_hmm_output(sub_hmm)$alignment_positions
+      #get the indices of alignemnt positions avaialble in both groups
+      intersection_pos = intersect(sub_pos, leftover_pos)
+      for (pos in seq_len(length(seqinr::s2c(alignment$seq[1])))) {
+        which_pos = which(intersection_pos == pos)
+        if (length(which_pos) > 0) {
+          pre_RE = c()
+          for (aa in seq_len(dim(sub_prob)[2])) {
+            sub_prob_pos = which(sub_pos == pos)
+            leftover_prob_pos = which(leftover_pos == pos)
+            pre_RE[aa] = sub_prob[sub_prob_pos, aa] * log(sub_prob[sub_prob_pos, aa] /
+                                                            leftover_prob[leftover_prob_pos, aa])
+          }
+          RE[i, pos] = sum(pre_RE, na.rm = T)
+        } else{
+          RE[i, pos] = 0
+        }
+      }
+    }else{
+      RE[i,] = 0
+    }
+  }
+  for (pos in seq_len(length(seqinr::s2c(alignment$seq[1])))) {
+    CRE[pos] = sum(RE[, pos])
+  }
+  return(CRE/max(CRE))
+}
+
+
 substitution_mtx <- function (matrix_name) {
   # function can read .txt file substitution matrix and convert it
   # into a list of alphabet (the range of letters in matrix) and
@@ -787,7 +1019,6 @@ landgraf_conservativity <-
       pre_dissim_mtx = substitution_mtx(matrix_name)
     }
     aligned_sequences_matrix = alignment2matrix(alignment = alignment)
-    aligned_sequences_matrix = toupper(aligned_sequences_matrix)
     dissim_mtx = D_matrix(pre_dissim_mtx)
     conservation = rep(NaN, dim(aligned_sequences_matrix)[2])
     status = 0
@@ -831,6 +1062,57 @@ landgraf_conservativity <-
     return(Landgraf_normalized_entropy)
   }
 
+calculate_pseudo_counts <- function(alignment, substitution_mtx = NULL) {
+  if (is.null(substitution_mtx)) {
+    gonnet = BALCONY::gonnet
+    substitution_mtx <-
+      apply(
+        X = gonnet[[2]],
+        MARGIN = 2,
+        FUN = function(X)
+          as.numeric(as.character(X))
+      )
+    substitution_mtx = exp(substitution_mtx)
+    colnames(substitution_mtx) <- gonnet[[1]]
+    rownames(substitution_mtx) <- gonnet[[1]]
+  }
+  #warning - other substitution matrices may have different symbols (like '*', or other letters)
+  mtx_alignment = alignment2matrix(alignment)
+  pseudoCounts = matrix(NA,
+                        nrow = length(append(Biostrings::AA_STANDARD, "-")),
+                        ncol = dim(mtx_alignment)[2])
+  B = apply(
+    X = mtx_alignment,
+    MARGIN = 2,
+    FUN = function(X)
+      (5 * length(unique(X)))
+  )
+  calc_ba <- function(column, B, substitution_mtx) {
+    pseudocounts = matrix(NA, nrow = length(append(Biostrings::AA_STANDARD, "-")), ncol = 1)
+    names(pseudocounts) <- append(Biostrings::AA_STANDARD, "-")
+    N = sum((table(column)))
+    AA = table(column)
+    for (i in names(pseudocounts)) {
+      to_sum <- rep(NA, length(AA))
+      cnt=1
+      for (j in names(AA)) {
+        to_sum[cnt] <- as.numeric(AA[j]/ N) * substitution_mtx[j, i]
+        cnt = cnt + 1
+      }
+      pseudocounts[i] <- sum(to_sum)
+    }
+    output = pseudocounts*(B/sum(pseudocounts))
+    return(output)
+  }
+  for (column in seq_len(ncol(mtx_alignment))) {
+    pseudoCounts[, column] <-
+      calc_ba(mtx_alignment[, column], B[column], substitution_mtx)
+  }
+  rownames(pseudoCounts) = append(Biostrings::AA_STANDARD, "-")
+  return(pseudoCounts)
+}
+
+
 
 # Structure analysis ------------------------------------------------------
 
@@ -902,7 +1184,7 @@ create_structure_seq <-
       seqs[[i]][structure_idx] = "S"
       aa_positions = which(seqinr::s2c(base_seq$sequence) != "-")
       just_align = alignment[[3]]
-      length_alignment = alignment_parameters(alignment)$col_no
+      length_alignment = align_params(alignment)$col_no
       structure[[i]] = rep("-", each = length_alignment)
       probability[[i]] = rep(NaN, each = length_alignment)
       if (prob_data == T) {
@@ -1384,3 +1666,38 @@ kolmogorov_smirnov_test <-
     }
     return(pval_mtx)
   }
+RealValET_conservativity <- function(alignment){
+  ET_groups = seq(1,length(alignment$nam))
+  dist <- seqinr::dist.alignment(alignment)
+  clust <- stats::hclust(dist, method = "average") #upgma
+  groups <- list() # groups for all nodes
+  for (i in ET_groups) {
+    groups[[i]] <- stats::cutree(clust,i)
+  }
+  aligned_matrix <- alignment2matrix(alignment = alignment)
+  shannon_group <- function(group){
+    counts = table(group)
+    len = length(group)
+    freq = counts/len
+    Shannon = -sum(freq * log(freq))
+  }
+  realValET <- function(column, group_tree){
+    nodes = length(group_tree) - 1
+    node_group = rep(0,nodes)
+    for (i in 1:nodes) {
+      temp = split(column, group_tree[[i]])
+      group = rep(0,length(temp))
+      for (j in 1:length(temp)) {
+        group[j] = shannon_group(temp[[j]])
+      }
+      node_group[i] = (1/i)*sum(group)
+    }
+    RO = 1 + sum(node_group)
+    return(RO)
+  }
+  x = rep(0,dim(aligned_matrix)[2])
+  for (i in 1:dim(aligned_matrix)[2]) {
+    x[i] = realValET(aligned_matrix[,i], groups)
+  }
+  return(x)
+}
